@@ -15,12 +15,12 @@ class UserCreateContract(Contract):
 
 
 class SendPasswordCreateContract(Contract):
-    username = PhoneNumberField()
+    user = serializers.IntegerField(min_value=1)
 
 
 class SendPasswordID(ID):
     _service = SERVICE
-    _app = 'user'
+    _app = 'users'
     _view_set = 'send_password'
 
 
