@@ -6,9 +6,10 @@ __all__ = ('PointRequestError', 'PointServerError', 'PointClientError', 'PointNo
            'Point', 'ObjectPoint', 'ContractPoint', 'ContractObjectPoint',
            'ListPointMixin', 'RetrievePointMixin', 'ResponseMixin', 'CreatePointMixin', 'UpdatePointMixin',
            'UploadFilePointMixin', 'ActionPointMixin',
-           'ID', 'Contract', 'PaginationContract'
+           'ID', 'Contract',
            )
 
+import re
 from typing import OrderedDict, Union
 
 from django.contrib.auth import get_user_model
@@ -16,13 +17,12 @@ from rest_framework.exceptions import ValidationError
 
 from expressmoney import status
 from expressmoney.api.cache import CacheMixin, CacheObjectMixin
-from expressmoney.api.contract import Contract, PaginationContract
+from expressmoney.api.contract import Contract
 from expressmoney.api.filter import FilterMixin
 from expressmoney.api.id import ID
 from expressmoney.api.utils import log
 from expressmoney.api.client import Request, Tasks
 
-
 User = get_user_model()
 
 
@@ -46,6 +46,10 @@ class ContractNotSet(PointError):
     pass
 
 
+class PaginationError(PointError):
+    pass
+
+
 class PointRequestError(PointError):
     default_url = None
     default_status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
@@ -73,6 +77,10 @@ class PointServerError(PointRequestError):
     pass
 
 
+class DatabasesPointServerError(PointServerError):
+    pass
+
+
 class PointClientError(PointRequestError):
     default_status_code = status.HTTP_400_BAD_REQUEST
     default_detail = 'Invalid payload.'
@@ -132,8 +140,8 @@ class Point:
         self._handle_error(self._response)
 
     @log
-    def _get(self) -> dict:
-        self._response = self._client.get()
+    def _get(self, url=None) -> dict:
+        self._response = self._client.get(url)
         self._handle_error(self._response)
         data = self._response.json()
         return data
@@ -157,7 +165,15 @@ class Point:
                     else:
                         raise PointClientError(self._client.url, response.status_code, response.json())
                 else:
-                    raise PointServerError(self._client.url, response.status_code, response.text)
+                    error = 'remaining connection slots are reserved for non-replication superuser connections'
+                    if re.search(rf'{error}', response.text):
+                        raise DatabasesPointServerError(self._client.url, response.status_code, error)
+                    elif re.search(r'OperationalError', response.text):
+                        raise DatabasesPointServerError(self._client.url, response.status_code,
+                                                        'Error related to database operation '
+                                                        '(disconnect, memory allocation).')
+                    else:
+                        raise PointServerError(self._client.url, response.status_code, response.text[:2048])
 
 
 class ObjectPoint(Point):
@@ -190,6 +206,25 @@ class ContractPoint(FilterMixin, CacheMixin, Point):
     _create_contract = None
     _sort_by = 'id'
 
+    def __init__(self,
+                 user: User,
+                 query_params: dict = None,
+                 is_async: bool = False,
+                 timeout: tuple = (30, 30),
+                 pagination_pages: int = 1,
+                 ):
+        super().__init__(user, query_params, is_async, timeout)
+        self._pagination_pages = pagination_pages
+        self.__next_pagination_page = None
+
+    @property
+    def _next_pagination_page(self):
+        return self.__next_pagination_page
+
+    @_next_pagination_page.setter
+    def _next_pagination_page(self, value):
+        self.__next_pagination_page = value
+
     def _get_sorted_data(self) -> tuple:
         if self._sort_by is None:
             raise SortByNotSet('Set key for sort or False')
@@ -198,43 +233,53 @@ class ContractPoint(FilterMixin, CacheMixin, Point):
         return tuple(sorted_data)
 
     def _get_validated_data(self):
-        data = self._handle_pagination(self._get())
+        data = self._get_data()
         contract = self._get_contract(data, True)
         validated_data = contract.validated_data
         if self._cache is None:
             self._cache = validated_data
         return validated_data
 
+    def _get_data(self):
+        page_data = self._get_page_data()
+        pages = self._pagination_pages
+        if not self._cache and pages is not None:
+            pages_read = 1
+            while (pages_read < pages or pages == 0) and self._next_pagination_page is not None:
+                page_data.extend(self._get_page_data(url=self._next_pagination_page))
+                pages_read += 1
+
+        return page_data
+
+    def _get_page_data(self, url=None) -> list:
+        get_data = self._get(url)
+        if self._cache or self._pagination_pages is None:
+            page_data = get_data
+            if not isinstance(page_data, list):
+                raise PaginationError('Endpoint pagination enable.')
+        else:
+            self._next_pagination_page = get_data.get('next')
+            page_data = get_data.get('results')
+            if page_data is None:
+                raise PaginationError('Endpoint pagination disable.')
+        return page_data
+
     def _get_contract(self, data, is_read: bool) -> Contract:
-        contract_class = self.__get_contract_class(is_read)
+        contract_class = self._get_contract_class(is_read)
         contract = contract_class(data=data, many=True if is_read else False)
-        self.__validate_contract(contract)
+        self._validate_contract(contract)
         return contract
 
-    def __get_contract_class(self, is_read: bool):
+    def _get_contract_class(self, is_read: bool):
         return self._read_contract if is_read else self._create_contract
 
-    def __validate_contract(self, contract):
+    def _validate_contract(self, contract):
         try:
             contract.is_valid(raise_exception=True)
         except ValidationError as e:
             self.flush_cache()
             raise ValidationError(e.detail)
 
-    @staticmethod
-    def _handle_pagination(data):
-        """Get current page and link on next page"""
-        if isinstance(data, list) or None in (data.get('count'), data.get('results')):
-            return data
-        pagination = {
-            'previous': data.get('previous'),
-            'next': data.get('next'),
-            'count': data.get('count'),
-        }
-        data = data.get('results')
-        data = [dict(**entity, pagination=pagination) for entity in data]
-        return data
-
 
 class ContractObjectPoint(CacheObjectMixin, ObjectPoint):
     """Endpoints for one object with validated data by contract"""
@@ -242,7 +287,7 @@ class ContractObjectPoint(CacheObjectMixin, ObjectPoint):
     _update_contract = None
 
     def _get_validated_data(self):
-        data = self._get()
+        data = self._get(url=None)
         if status.is_not_found(data):
             raise PointNotFound404
         contract = self._get_contract(data, True)
